#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file CloudDB_3DImageRead.py
 @brief Acquisition information from the mongo database
 @date $Date$

 @author EAST JAPAN ACCOUNTING CENTER CO.,LTD.

 BSD 3-clause "New" or "Revised" License

"""
import sys
import time

# Import RTM module
import RTC
import OpenRTM_aist

# add to code
import os
import os.path
import copy
import datetime
import dateutil.parser
import pymongo

sys.path.append(".")

""" message definition """
messege_text_01 = ("onInitialize")
messege_text_02 = ("onFinalize")
messege_text_03 = ("onActivated")
messege_text_04 = ("onDeactivated")
messege_text_05 = ("Heart beat by CloudDB_3DImageRead")
messege_text_06 = (
    "Error was detected. \
Please reset the CloudDB_3DImageRead RTC by the RT System Editor.")
messege_text_07 = ("Input database search condition")
messege_text_08 = ("!write fail - DB connection status")
messege_text_09 = ("!write okay - DB connection status")
messege_text_10 = ("Date data is wrong. Please confirm.")
messege_text_11 = ("Search conditions are wrong. Please confirm.")
messege_text_12 = ("!write fail - file names list")
messege_text_13 = ("!write okay - file names list")
messege_text_14 = (
    "There is no image data \
that meet the search conditions.")
messege_text_15 = ("Connected to the MongoDB")
messege_text_16 = ("Not connected to the MongoDB")
messege_text_17 = ("Could not connect to the MongoDB.")
messege_text_18 = ("The specified number of lists exceeded.")

""" const definition """
YEAR_MIN = 0     # year_min
MON__MIN = 1     # mon__min
DAY__MIN = 2     # day__min
HOUR_MIN = 3     # hour_min
MIN__MIN = 4     # min__min
SEC__MIN = 5     # sec__min

YEAR_MAX = 6     # year_max
MON__MAX = 7     # mon__max
DAY__MAX = 8     # day__max
HOUR_MAX = 9     # hour_max
MIN__MAX = 10    # min__max
SEC__MAX = 11    # sec__max

ROBOT_TYPE = 12

LIST_COUNT_INIT = (0)
LIST_COUNT_MAX = (500)

MONGO_DB_NAME = ("MylocalDB")
DB_COLLECTION = ("User_col")

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
clouddb_3dimageread_spec = ["implementation_id", "CloudDB_3DImageRead",
                            "type_name",         "CloudDB_3DImageRead",
                            "description",       "Acquisition information \
                                                      from the mongo database",
                            "version",           "1.0",
                            "vendor",            "EAC",
                            "category",          "Database",
                            "activity_type",     "STATIC",
                            "max_instance",      "1",
                            "language",          "Python",
                            "lang_type",         "SCRIPT",
                            ""]
# </rtc-template>

##
# @class CloudDB_3DImageRead
# @brief Acquisition information from the mongo database
#
# Acquisition information from the MongoDB.
#
# -
#
# See "functional specification document_CloudDB_3DImageRead"
#
# -
#
#


class CloudDB_3DImageRead(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_in_condition = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        Port to input of file name-list the search condition.
         - Type: RTC::TimedStringSeq
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: One cycle per second.
        """
        self._db_search_conditionIn = OpenRTM_aist.InPort(
            "db_search_condition", self._d_in_condition)
        self._d_out_filelist = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        Output the file-name-list of search results.
         - Type: RTC::TimedStringSeq
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: One cycle per second.
        """
        self._file_list_outOut = OpenRTM_aist.OutPort(
            "file_list_out", self._d_out_filelist)
        self._d_out_connection_sts = RTC.TimedString(RTC.Time(0, 0), "")
        """
        Provide MongoDB connection status.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: When connect to the database.
         - Operation Cycle: One cycle per second.
        """
        self._DBConnectionStatusServOut = OpenRTM_aist.OutPort(
            "DBConnectionStatusServ", self._d_out_connection_sts)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>

    ##
    # Initialization of CloudDB_3DImageRead RTC
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    # @pre -
    # @post -
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers
        self.addInPort("db_search_condition", self._db_search_conditionIn)

        # Set OutPort buffers
        self.addOutPort("file_list_out", self._file_list_outOut)
        self.addOutPort("DBConnectionStatusServ",
                        self._DBConnectionStatusServOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        # add to code
        print messege_text_01
        self.onInitializeFunc()
        self._MongoDBConnection()

        return RTC.RTC_OK

        ##
        # Finalization of CloudDB_3DImageRead RTC
        #
        # The finalize action (on ALIVE->END transition)
        # formaer rtc_exiting_entry()
        #
        # @return RTC::ReturnCode_t

        # @pre -
        # @post -
        #
    def onFinalize(self):

        # add to code
        print messege_text_02

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Activate the CloudDB_3DImageRead RTC
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onActivated(self, ec_id):

        # add to code
        print messege_text_03
        self.onActivatedFunc()

        return RTC.RTC_OK

        ##
        # Deactivate the CloudDB_3DImageRead RTC
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onDeactivated(self, ec_id):

        # add to code
        print messege_text_04
        self.onDeactivatedFunc()

        return RTC.RTC_OK

        ##
        # Periodic task of CloudDB_3DImageRead RTC.
        # This task connecting to MongoDB. And access control for MongoDB.
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onExecute(self, ec_id):

        # add to code
        self.onExecuteFunc()
        print messege_text_05

        return RTC.RTC_OK

        ##
        # onAborting the CloudDB_3DImageRead RTC
        #
        # The aborting action when main logic error occurred.
        # former rtc_aborting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onAborting(self, ec_id):

        # add to code
        print messege_text_06
        self.onAbortingFunc()

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Reset of CloudDB_3DImageRead RTC
        #
        # The reset action that is invoked resetting
        # This is same but different the former rtc_init_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onReset(self, ec_id):

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    # add to code
    # =========================================================================
    # Function Name   :onExecuteFunc
    # Description     :This function is the main function that is executed
    #                  from "onExecute" function.
    # Argument        :input-port of the RTC
    # Return          :nothing special
    # =========================================================================
    def onExecuteFunc(self):

        # ==================================================================
        # check DB connection status
        self.check_DBconnection_status()

        # ==================================================================
        # check database search condition
        if self._db_search_conditionIn.isNew():
            print messege_text_07
            search_data = self._db_search_conditionIn.read()
            self._CreateList_of_FileName(search_data)

    # =========================================================================
    # Function Name   :onActivatedFunc
    # Description     :This function is an "onActivated process"
    #                  run-time initialization process.
    # Argument        :onAborting process detection inf,
    # Return          :nothing special
    # =========================================================================
    def onActivatedFunc(self):
        if self.onAbortingDetect == 1:
            self.onInitializeFunc()
            self._MongoDBConnection()
        else:
            self.onAbortingDetect = 0
            self.timer_count = 0

    # =========================================================================
    # Function Name   :onDeactivatedFunc
    # Description     :This function is an "onDeactivated process"
    #                  run-time initialization process.
    # Argument        :connection status inf
    # Return          :nothing special
    # =========================================================================
    def onDeactivatedFunc(self):
        self.timer_count = 0

    # =========================================================================
    # Function Name   :onAbortingFunc
    # Description     :This function is an "onAborting process"
    #                  run-time initialization process.
    # Argument        :nothing special
    # Return          :nothing special
    # =========================================================================
    def onAbortingFunc(self):
        self.onAbortingDetect = 1

    # =========================================================================
    # Function Name   :onInitializeFunc
    # Description     :This function is an "onInitialize process"
    #                  run-time initialization process.
    # Argument        :nothing special
    # Return          :nothing special
    # =========================================================================
    def onInitializeFunc(self):

        # Year month day
        self.year_min = 0
        self.mon__min = 0
        self.day__min = 0
        self.hour_min = 0
        self.min__min = 0
        self.sec__min = 0

        self.year_max = 0
        self.mon__max = 0
        self.day__max = 0
        self.hour_max = 0
        self.min__max = 0
        self.sec__max = 0

        self.robot_type = ""

        # output data
        self._d_out_filelist.data = []
        self._d_out_filelist.tm.sec = 0
        self._d_out_filelist.tm.nsec = 0

        self._d_out_connection_sts.data = ""
        self._d_out_connection_sts.tm.sec = 0
        self._d_out_connection_sts.tm.nsec = 0

        # work data
        self.listout = []

        self.date_time_sec = 0
        self.date_time_nsec = 0

        self.onAbortingDetect = 0
        self.db_connection_sts = False
        self.timer_count = 0

    # =========================================================================
    # Function Name   :_MongoDBConnection
    # Description     :This function execute the connection
    #                  of MongoDB and the RTC.
    # Argument        :nothing special
    # Return          :nothing special
    # =========================================================================
    def _MongoDBConnection(self):

        self.db_connection_sts = False
        try:
            self.client = pymongo.MongoClient('localhost', 27017)
        except Exception as err:
            print messege_text_17
            print err.message
        else:
            self.db_connection_sts = True
            self.db = self.client[MONGO_DB_NAME]
            self.col = self.db[DB_COLLECTION]
            print ("db name is = " + self.db.name)
            print ("collection name is = " + self.col.name)

    # =========================================================================
    # Function Name   :check_DBconnection_status
    # Description     :This function to check the MongoDB connection.
    # Argument        :Number of documents
    # Return          :nothing special
    # =========================================================================
    def check_DBconnection_status(self):

        self.timer_count = self.timer_count + 1
        if self.timer_count > 9:
            self.timer_count = 0

            health_check = 0
            self.db_connection_sts = False
            self._d_out_connection_sts.data = "0"
            try:
                health_check = self.col.find().count()
            except Exception as err:
                print messege_text_16
                print err.message
            else:
                if health_check >= 1:
                    self.db_connection_sts = True
                    # connected to the MongoDB
                    self._d_out_connection_sts.data = "1"
                    print messege_text_15
                else:
                    print messege_text_16

            self.write_DBconnection_status()

    # =========================================================================
    # Function Name   :write_DBconnection_status
    # Description     :This function outputs
    #                  the MongoDB connection information to the port.
    # Argument        :nothing special
    # Return          :nothing special
    # =========================================================================
    def write_DBconnection_status(self):

        self._DateTimeSetting()
        self._d_out_connection_sts.tm.sec = copy.deepcopy(
            self.date_time_sec)
        self._d_out_connection_sts.tm.nsec = copy.deepcopy(
            self.date_time_nsec)

        ret = self._DBConnectionStatusServOut.write()
        if not ret:
            print messege_text_08
        else:
            print messege_text_09

    # =========================================================================
    # Function Name   :_CreateList_of_FileName
    # Description     :This function is the main function of database search.
    # Argument        :database search conditions
    # Return          :nothing special
    # =========================================================================
    def _CreateList_of_FileName(self, search_value):

        if self.get_db_connection_sts():
            if self._SearchDataSetting(search_value):
                if self._SearchDataFind():
                    self._Search_Resultslist()

    # =========================================================================
    # Function Name   :_SearchDataSetting
    # Description     :This function sets the DB search conditions
    #                  to the static variable.
    # Argument        :database search conditions
    # Return          :information of normal end(True) or abnormal end(False).
    # =========================================================================
    def _SearchDataSetting(self, search_value):
        return_value = False

        if self.check_string_of_datetime(search_value):
            if self.check_limits_of_datedata(search_value):
                if self.validity_check_of_datetime(search_value):
                    self._DateDataSetting(search_value)
                    # robot type
                    self.robot_type = copy.deepcopy(
                        search_value.data[ROBOT_TYPE])
                    return_value = True
                else:
                    print messege_text_10
            else:
                print messege_text_10

        return return_value

    # =========================================================================
    # Function Name   :_SearchDataFind
    # Description     :This function execute the database search.
    # Argument        :database search conditions
    # Return          :information of normal end(True) or abnormal end(False).
    # =========================================================================
    def _SearchDataFind(self):
        return_value = False
        flag = False

        date_min = datetime.datetime(
            self.year_min, self.mon__min, self.day__min,
            self.hour_min, self.min__min, self.sec__min, 0)

        date_max = datetime.datetime(
            self.year_max, self.mon__max, self.day__max,
            self.hour_max, self.min__max, self.sec__max, 0)

        dmin = date_min.strftime("%Y-%m-%d %H:%M:%S")
        dmax = date_max.strftime("%Y-%m-%d %H:%M:%S")

        if self.robot_type == "drone1":
            ret = self.col.find({"robot_id.string": "drone1", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "drone2":
            ret = self.col.find({"robot_id.string": "drone2", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "drone3":
            ret = self.col.find({"robot_id.string": "drone3", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "drone4":
            ret = self.col.find({"robot_id.string": "drone4", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "drone5":
            ret = self.col.find({"robot_id.string": "drone5", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "drone6":
            ret = self.col.find({"robot_id.string": "drone6", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "spider1":
            ret = self.col.find({"robot_id.string": "spider1", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "spider2":
            ret = self.col.find({"robot_id.string": "spider2", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "spider3":
            ret = self.col.find({"robot_id.string": "spider3", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "enryu1":
            ret = self.col.find({"robot_id.string": "enryu1", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "enryu2":
            ret = self.col.find({"robot_id.string": "enryu2", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "enryu3":
            ret = self.col.find({"robot_id.string": "enryu3", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "gorone1":
            ret = self.col.find({"robot_id.string": "gorone1", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "gorone2":
            ret = self.col.find({"robot_id.string": "gorone2", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "gorone3":
            ret = self.col.find({"robot_id.string": "gorone3", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "gorone4":
            ret = self.col.find({"robot_id.string": "gorone4", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "gorone5":
            ret = self.col.find({"robot_id.string": "gorone5", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        elif self.robot_type == "gorone6":
            ret = self.col.find({"robot_id.string": "gorone6", "time.string": {
                                "$gte": dmin, "$lte": dmax}})
            flag = True
        # add robot_type
        else:
            print messege_text_11

        if flag:
            if self.validity_check_of_filename_list(ret):
                # file names list exist
                return_value = True

        return return_value

    # =========================================================================
    # Function Name   :_Search_Resultslist
    # Description     :This function outputs the file names list to port.
    # Argument        :file names list
    # Return          :port output results
    # =========================================================================
    def _Search_Resultslist(self):

        del self._d_out_filelist.data[:]
        # batch conversion to a string.
        listout_str = map(str, self.listout)
        self._d_out_filelist.data = copy.deepcopy(listout_str)

        self._DateTimeSetting()
        self._d_out_filelist.tm.sec = copy.deepcopy(self.date_time_sec)
        self._d_out_filelist.tm.nsec = copy.deepcopy(self.date_time_nsec)

        ret = self._file_list_outOut.write()
        if not ret:
            print messege_text_12
        else:
            print messege_text_13

    # =========================================================================
    # Function Name   :_DateTimeSetting
    # Description     :Set the time information
    # Argument        :nothing special
    # Return          :nothing special
    # =========================================================================
    def _DateTimeSetting(self):

        timenow = datetime.datetime.now()
        daysec = (timenow.day * 24 * 3600)
        hoursec = (timenow.hour * 3600)
        minsec = (timenow.minute * 60)

        self.date_time_sec = (
            daysec + hoursec + minsec + timenow.second)
        self.date_time_nsec = (timenow.microsecond * 1000)

    # =========================================================================
    # Function Name   :check_string_of_datetime
    # Description     :This function confirm the string of dates and time.
    # Argument        :date, time
    # Return          :information of normal end(True) or abnormal end(False).
    # =========================================================================
    def check_string_of_datetime(self, dt_val):
        return_value = False

        date_time_min = dt_val.data[YEAR_MIN] \
            + "/" + dt_val.data[MON__MIN] \
            + "/" + dt_val.data[DAY__MIN] \
            + " " + dt_val.data[HOUR_MIN] \
            + ":" + dt_val.data[MIN__MIN] \
            + ":" + dt_val.data[SEC__MIN]

        date_time_max = dt_val.data[YEAR_MAX] \
            + "/" + dt_val.data[MON__MAX] \
            + "/" + dt_val.data[DAY__MAX] \
            + " " + dt_val.data[HOUR_MAX] \
            + ":" + dt_val.data[MIN__MAX] \
            + ":" + dt_val.data[SEC__MAX]

        try:
            date_time = dateutil.parser.parse(date_time_min)
        except ValueError:
            print messege_text_10
        except Exception as err:
            print err.message
        else:
            try:
                date_time = dateutil.parser.parse(date_time_max)
            except ValueError:
                print messege_text_10
            except Exception as err:
                print err.message
            else:
                return_value = True

        return return_value

    # =========================================================================
    # Function Name   :check_limits_of_datedata
    # Description     :This function confirm the range of dates.
    # Argument        :date
    # Return          :information of normal end(True) or abnormal end(False).
    # =========================================================================
    def check_limits_of_datedata(self, dt_val):
        return_value = False

        year_min = copy.deepcopy(int(dt_val.data[YEAR_MIN]))
        year_max = copy.deepcopy(int(dt_val.data[YEAR_MAX]))

        if (year_min > 2015) and (year_max > 2015):
            return_value = True

        return return_value

    # =========================================================================
    # Function Name   :validity_check_of_datetime
    # Description     :This function to check
    #                  whether the valid information of the date and time.
    # Argument        :date, time
    # Return          :information of normal end(True) or abnormal end(False).
    # =========================================================================
    def validity_check_of_datetime(self, dt_val):
        return_value = False

        year_min = copy.deepcopy(int(dt_val.data[YEAR_MIN]))
        mon__min = copy.deepcopy(int(dt_val.data[MON__MIN]))
        day__min = copy.deepcopy(int(dt_val.data[DAY__MIN]))
        hour_min = copy.deepcopy(int(dt_val.data[HOUR_MIN]))
        min__min = copy.deepcopy(int(dt_val.data[MIN__MIN]))
        sec__min = copy.deepcopy(int(dt_val.data[SEC__MIN]))

        year_max = copy.deepcopy(int(dt_val.data[YEAR_MAX]))
        mon__max = copy.deepcopy(int(dt_val.data[MON__MAX]))
        day__max = copy.deepcopy(int(dt_val.data[DAY__MAX]))
        hour_max = copy.deepcopy(int(dt_val.data[HOUR_MAX]))
        min__max = copy.deepcopy(int(dt_val.data[MIN__MAX]))
        sec__max = copy.deepcopy(int(dt_val.data[SEC__MAX]))

        date_min = datetime.date(
            year_min, mon__min, day__min)
        date_max = datetime.date(
            year_max, mon__max, day__max)

        datetime_min = datetime.time(
            hour_min, min__min, sec__min)
        datetime_max = datetime.time(
            hour_max, min__max, sec__max)

        if date_max >= date_min:
            if date_max == date_min:
                if datetime_max > datetime_min:
                    return_value = True
            else:
                return_value = True

        return return_value

    # =========================================================================
    # Function Name   :_DateDataSetting
    # Description     :Set the date information
    # Argument        :nothing special
    # Return          :nothing special
    # =========================================================================
    def _DateDataSetting(self, dt_val):
        # Year month day
        self.year_min = copy.deepcopy(int(dt_val.data[YEAR_MIN]))
        self.mon__min = copy.deepcopy(int(dt_val.data[MON__MIN]))
        self.day__min = copy.deepcopy(int(dt_val.data[DAY__MIN]))
        self.hour_min = copy.deepcopy(int(dt_val.data[HOUR_MIN]))
        self.min__min = copy.deepcopy(int(dt_val.data[MIN__MIN]))
        self.sec__min = copy.deepcopy(int(dt_val.data[SEC__MIN]))

        self.year_max = copy.deepcopy(int(dt_val.data[YEAR_MAX]))
        self.mon__max = copy.deepcopy(int(dt_val.data[MON__MAX]))
        self.day__max = copy.deepcopy(int(dt_val.data[DAY__MAX]))
        self.hour_max = copy.deepcopy(int(dt_val.data[HOUR_MAX]))
        self.min__max = copy.deepcopy(int(dt_val.data[MIN__MAX]))
        self.sec__max = copy.deepcopy(int(dt_val.data[SEC__MAX]))

    # =========================================================================
    # Function Name   :validity_check_of_filename_list
    # Description     :This function to check
    #                  whether the valid information of the file names list.
    # Argument        :file name list
    # Return          :information of normal end(True) or abnormal end(False).
    # =========================================================================
    def validity_check_of_filename_list(self, search_results):
        return_value = False

        del self.listout[:]
        list_count = LIST_COUNT_INIT
        pass_flag = False

        for doc in search_results:
            check_path = doc["image_path"]["string"]
            if self.check_file_exists(check_path) is True:
                if list_count < LIST_COUNT_MAX:
                    self.listout.append(check_path)
                    list_count = list_count + 1
                    pass_flag = True
                else:
                    if list_count == LIST_COUNT_MAX:
                        print messege_text_18

        if pass_flag:
            return_value = True
        else:
            print messege_text_14

        return return_value

    # =========================================================================
    # Function Name   :check_file_exists
    # Description     :This function checks whether a file exists.
    # Argument        :file path
    # Return          :information of normal end or abnormal end.
    # =========================================================================
    def check_file_exists(self, value):
        result = False
        try:
            if os.path.exists(value) is True:
                result = True
        except Exception as err:
            print err.message
        else:
            return result

        return result

    # =========================================================================
    # Function Name   :get_db_connection_sts
    # Description     :This function returns the database connection status.
    # Argument        :nothing special
    # Return          :information of normal end(True) or abnormal end(False).
    # =========================================================================
    def get_db_connection_sts(self):
        return_value = False

        if self.db_connection_sts:
            return_value = True

        return return_value

    # =========================================================================


def CloudDB_3DImageReadInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=clouddb_3dimageread_spec)
    manager.registerFactory(profile,
                            CloudDB_3DImageRead,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    CloudDB_3DImageReadInit(manager)

    # Create a component
    comp = manager.createComponent("CloudDB_3DImageRead")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()
