#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file ImageLoader.py
 @brief image acquisition component
 @date $Date$

 @author EAST JAPAN ACCOUNTING CENTER CO.,LTD.

 BSD 3-clause "New" or "Revised" License

"""
import sys
import time

# Import RTM module
import RTC
import OpenRTM_aist

# add to code
import os
import os.path
import copy
import numpy as np
import cv2

sys.path.append(".")

""" message definition """
messege_text_01 = ("onInitialize")
messege_text_02 = ("onFinalize")
messege_text_03 = ("onActivated")
messege_text_04 = ("onDeactivated")
messege_text_05 = ("Detected the multi-port.")
messege_text_06 = ("Detected the single-port.")
messege_text_07 = ("Heart beat by onExecute of ImageLoader.")
messege_text_08 = (
    "Error was detected. \
Please reset the ImageLoader RTC by the RT System Editor.")
messege_text_09 = ("single write success.")
messege_text_10 = ("This software supports 24bpp of 3-channel.")
messege_text_11 = ("multi-data output success.")
messege_text_12 = ("Reading of Image file failed.")
messege_text_13 = ("This path information is not valid.")
messege_text_14 = ("Unsupported format.")
messege_text_15 = (
    "Please check channel, this software supports 3-ch.")
messege_text_16 = (
    "Parameter is wrong. please check parameter.")
messege_text_17 = ("!write error")
messege_text_18 = ("This image a greyscale image or mono image.")
messege_text_19 = ("These images included grayscale images.")
messege_text_20 = ("Check whether this images 24bpp.")
messege_text_21 = ("This software is not supported on alpha-channel.")

""" const definition """
KIND_S = 0     # Single
KIND_L = 1     # Left
KIND_C = 2     # Center
KIND_R = 3     # Right
KIND_MIN = KIND_S
KIND_MAX = (KIND_R + 1)

# Import Service implementation class
# <rtc-template block="service_impl">

# </rtc-template>

# Import Service stub modules
# <rtc-template block="consumer_import">
# </rtc-template>


# This module's spesification
# <rtc-template block="module_spec">
imageloader_spec = ["implementation_id", "ImageLoader",
                    "type_name",         "ImageLoader",
                    "description",       "image acquisition component",
                    "version",           "1.0",
                    "vendor",            "EAC",
                    "category",          "image-acquisition",
                    "activity_type",     "STATIC",
                    "max_instance",      "1",
                    "language",          "Python",
                    "lang_type",         "SCRIPT",
                    ""]
# </rtc-template>

##
# @class ImageLoader
# @brief image acquisition component
#
# This component uses the OpenCV library to load images.
#
# -
#
# See "functional specification document_Image_Loader"
#
# http://docs.opencv.org/
#
#


class ImageLoader(OpenRTM_aist.DataFlowComponentBase):

    ##
    # @brief constructor
    # @param manager Maneger Object
    #
    def __init__(self, manager):
        OpenRTM_aist.DataFlowComponentBase.__init__(self, manager)

        self._d_in_path = RTC.TimedString(RTC.Time(0, 0), "")
        """
        This port inputs a single absolute path.
         - Type: RTC::TimedString
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: one-cycle per second.
        """
        self._ImagePathIn = OpenRTM_aist.InPort("ImagePath", self._d_in_path)
        self._d_in_path_multi = RTC.TimedStringSeq(RTC.Time(0, 0), [])
        """
        This port inputs multiple absolute paths.
         - Type: RTC::TimedStringSeq
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: one-cycle per second.
        """
        self._ImagePathMultiIn = OpenRTM_aist.InPort(
            "ImagePathMulti", self._d_in_path_multi)
        self._d_out_image = RTC.CameraImage(
            RTC.Time(0, 0), 0, 0, 0, "", 0.0, [])
        """
        This port outputs the CameraImage type data.
         - Type: RTC::CameraImage
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: one-cycle per second.
        """
        self._ImageDataOut = OpenRTM_aist.OutPort(
            "ImageData", self._d_out_image)
        self._d_out_image_multi = RTC.MultiCameraImages(RTC.Time(0, 0), [])
        """
        This port outputs the MultiCameraImages type data.
         - Type: RTC::MultiCameraImages
         - Number: 1
         - Semantics: -
         - Unit: -
         - Frequency: Caused by user actions.
         - Operation Cycle: one-cycle per second.
        """
        self._ImageDataMultiOut = OpenRTM_aist.OutPort(
            "ImageDataMulti", self._d_out_image_multi)

        # initialize of configuration-data.
        # <rtc-template block="init_conf_param">

        # </rtc-template>

    ##
    # Initialization of the ImageLoader RTC
    #
    # The initialize action (on CREATED->ALIVE transition)
    # formaer rtc_init_entry()
    #
    # @return RTC::ReturnCode_t
    #
    # @pre -
    # @post -
    #
    def onInitialize(self):
        # Bind variables and configuration variable

        # Set InPort buffers
        self.addInPort("ImagePath", self._ImagePathIn)
        self.addInPort("ImagePathMulti", self._ImagePathMultiIn)

        # Set OutPort buffers
        self.addOutPort("ImageData", self._ImageDataOut)
        self.addOutPort("ImageDataMulti", self._ImageDataMultiOut)

        # Set service provider to Ports

        # Set service consumers to Ports

        # Set CORBA Service Ports

        # add to code
        print messege_text_01
        self.img = []

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The finalize action (on ALIVE->END transition)
    #    # formaer rtc_exiting_entry()
    #    #
    #    # @return RTC::ReturnCode_t
    #
    #    #
    # def onFinalize(self):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The startup action when ExecutionContext startup
    #    # former rtc_starting_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onStartup(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The shutdown action when ExecutionContext stop
    #    # former rtc_stopping_entry()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onShutdown(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Activate the ImageLoader RTC
        #
        # The activated action (Active state entry action)
        # former rtc_active_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onActivated(self, ec_id):
        # add to code
        print messege_text_03

        self.height = 240
        self.width = 320
        self.imageformat = "bitmap"

        self.img.append(np.zeros(
            (self.height, self.width, 3), dtype=np.uint8))
        self.img.append(np.zeros(
            (self.height, self.width, 3), dtype=np.uint8))
        self.img.append(np.zeros(
            (self.height, self.width, 3), dtype=np.uint8))
        self.img.append(np.zeros(
            (self.height, self.width, 3), dtype=np.uint8))

        return RTC.RTC_OK

        ##
        # Deactivate the ImageLoader RTC
        #
        # The deactivated action (Active state exit action)
        # former rtc_active_exit()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onDeactivated(self, ec_id):
        # add to code
        print messege_text_04
        del self.img[:]

        return RTC.RTC_OK

        ##
        # Periodic task of the ImageLoader RTC
        #
        # The execution action that is invoked periodically
        # former rtc_active_do()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onExecute(self, ec_id):

        # add to code
        # =========================================================================================
        # check multi-port
        if self._ImagePathMultiIn.isNew():
            print messege_text_05
            self.case_image_file_multi()

        # =========================================================================================
        else:
            # check single-port
            if self._ImagePathIn.isNew():
                print messege_text_06
                self.case_image_file_single()

        # =========================================================================================

        print messege_text_07

        return RTC.RTC_OK

        ##
        # onAborting the ImageLoader RTC
        #
        # The aborting action when main logic error occurred.
        # former rtc_aborting_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onAborting(self, ec_id):
        # add to code
        print messege_text_08

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The error action in ERROR state
    #    # former rtc_error_do()
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onError(self, ec_id):
    #
    #    return RTC.RTC_OK

        ##
        # Reset initialization of the ImageLoader RTC
        #
        # The reset action that is invoked resetting
        # This is same but different the former rtc_init_entry()
        #
        # @param ec_id target ExecutionContext Id
        #
        # @return RTC::ReturnCode_t
        #
        # @pre -
        # @post -
        #
    def onReset(self, ec_id):

        return RTC.RTC_OK

    #    ##
    #    #
    #    # The state update action that is invoked after onExecute() action
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #

    #    #
    # def onStateUpdate(self, ec_id):
    #
    #    return RTC.RTC_OK

    #    ##
    #    #
    #    # The action that is invoked when execution context's rate is changed
    #    # no corresponding operation exists in OpenRTm-aist-0.2.0
    #    #
    #    # @param ec_id target ExecutionContext Id
    #    #
    #    # @return RTC::ReturnCode_t
    #    #
    #    #
    # def onRateChanged(self, ec_id):
    #
    #    return RTC.RTC_OK

    # function definition
    # =============================================================================================
    # Function Name   :case_image_file_single
    # Description     :This function will confirm the data of the single-port.
    # Argument        :Single path information
    # Return          :nothing special
    # =============================================================================================
    def case_image_file_single(self):
        filepath_single = self._ImagePathIn.read()
        self.ospath_single = os.path.abspath(filepath_single.data)

        if self.validity_check_of_path(self.ospath_single):
            self.image_file_analysis_single(self.ospath_single)

    # =============================================================================================
    # Function Name   :case_image_file_multi
    # Description     :This function will confirm the data of the multi-port.
    # Argument        :Multi path information
    # Return          :nothing special
    # =============================================================================================
    def case_image_file_multi(self):
        filepath_multi = self._ImagePathMultiIn.read()
        self.ospath_multi_l = os.path.abspath(filepath_multi.data[0])
        self.ospath_multi_c = os.path.abspath(filepath_multi.data[1])
        self.ospath_multi_r = os.path.abspath(filepath_multi.data[2])

        if self.validity_check_of_path(self.ospath_multi_l):
            if self.validity_check_of_path(self.ospath_multi_c):
                if self.validity_check_of_path(self.ospath_multi_r):
                    self.image_file_analysis_multi(
                        self.ospath_multi_l,
                        self.ospath_multi_c, self.ospath_multi_r)

    # =============================================================================================
    # Function Name   :image_file_analysis_single
    # Description     :This function to analysis of a single image.
    # Argument        :Single path information
    # Return          :CameraImage
    # =============================================================================================
    def image_file_analysis_single(self, path_single):

        if self.validity_check_of_image(path_single, KIND_S):
            # check color
            if len(self.img[KIND_S].shape) == 3:
                print ("channel single = %d" % self.img[KIND_S].shape[2])

                # check of channels number and dtype
                if (self.img[KIND_S].shape[2] == 3) and \
                   (self.img[KIND_S].dtype == "uint8"):
                    # this case 24 bpp
                    self.image_data_setting_single(
                                        path_single, 24, self.img[KIND_S])

                else:
                    if self.img[KIND_S].shape[2] == 4:
                        # this case alpha-channel
                        print messege_text_10
                        print messege_text_21
                    else:
                        print messege_text_10

            else:
                print messege_text_18
                # this case greyscale or mono
                self.image_data_setting_single(
                                        path_single, 8, self.img[KIND_S])

    # =============================================================================================
    # Function Name   :image_file_analysis_multi
    # Description     :This function to analysis of multiple images.
    # Argument        :Multi path information(left, center, right)
    # Return          :Multi-CameraImages
    # =============================================================================================
    def image_file_analysis_multi(self, path_l, path_c, path_r):

        if self.validity_check_of_image(path_l, KIND_L) and \
           self.validity_check_of_image(path_c, KIND_C) and \
           self.validity_check_of_image(path_r, KIND_R):

            if self.validity_check_of_channel(
                    self.img[KIND_L], self.img[KIND_C], self.img[KIND_R]):

                multi_image = RTC.MultiCameraImages(
                    RTC.Time(0, 0), [])
                self.image_work_l = RTC.CameraImage(
                    RTC.Time(0, 0), 0, 0, 0, "", 0.0, [])
                self.image_work_c = RTC.CameraImage(
                    RTC.Time(0, 0), 0, 0, 0, "", 0.0, [])
                self.image_work_r = RTC.CameraImage(
                    RTC.Time(0, 0), 0, 0, 0, "", 0.0, [])

                del multi_image.images[:]
                del self._d_out_image_multi.images[:]

                self.image_data_setting_l(path_l, self.img[KIND_L])
                multi_image.images.append(self.image_work_l)

                self.image_data_setting_c(path_c, self.img[KIND_C])
                multi_image.images.append(self.image_work_c)

                self.image_data_setting_r(path_r, self.img[KIND_R])
                multi_image.images.append(self.image_work_r)

                self._d_out_image_multi.tm = multi_image.tm
                self._d_out_image_multi.images = multi_image.images

                self.ret_m = self._ImageDataMultiOut.write()
                if not self.ret_m:
                    print messege_text_17
                else:
                    print messege_text_11
            else:
                pass

    # =============================================================================================
    # Function Name   :validity_check_of_channel
    # Description     :This function to confirmation the validity
    #                 :of the channel.
    # Argument        :image
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def validity_check_of_channel(self, img_l, img_c, img_r):
        return_value = False

        # check color
        if (len(img_l.shape) == 3) and (len(img_c.shape) == 3) and \
           (len(img_r.shape) == 3):

            print ("channel left = %d" % img_l.shape[2]) + \
                  (", center = %d" % img_c.shape[2]) + \
                  (", right = %d" % img_r.shape[2])

            # check of channels number
            if (img_l.shape[2] == 3) and (img_c.shape[2] == 3) and \
               (img_r.shape[2] == 3):

                print messege_text_20

                # check dtype
                if (img_l.dtype == "uint8") and (img_c.dtype == "uint8") and \
                   (img_r.dtype == "uint8"):

                    return_value = True
                else:
                    print messege_text_10
            else:
                if (img_l.shape[2] == 4) or (img_c.shape[2] == 4) or \
                   (img_r.shape[2] == 4):
                    # this case alpha-channel
                    print messege_text_15
                    print messege_text_21
                else:
                    print messege_text_15
        else:
            # this case greyscale or mono
            print messege_text_19

        return return_value

    # =============================================================================================
    # Function Name   :validity_check_of_image
    # Description     :This function to confirmation the validity
    #                 :of the image.
    # Argument        :Path information, image kind
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def validity_check_of_image(self, path_value, im_kind):
        return_value = False

        img_temp = cv2.imread(path_value, cv2.IMREAD_UNCHANGED)
        if img_temp is None:
            print messege_text_12
        else:
            img_temp_resize = cv2.resize(
                img_temp, (self.width, self.height))

            if (im_kind >= KIND_MIN) and (im_kind < KIND_MAX):
                return_value = True
                self.img[im_kind] = img_temp_resize.copy()

            else:
                print messege_text_16

        return return_value

    # =============================================================================================
    # Function Name   :image_data_setting_single
    # Description     :This function is to set the CameraImage type data.
    # Argument        :path information, channel, image data
    # Return          :nothing special
    # =============================================================================================
    def image_data_setting_single(self, path_value, channel, img):
        return_temp = False

        self._d_out_image.height = copy.deepcopy(self.height)
        self._d_out_image.width = copy.deepcopy(self.width)
        self._d_out_image.bpp = copy.deepcopy(channel)
        return_temp = self.check_extension(path_value)
        self._d_out_image.format = copy.deepcopy(self.imageformat)
        self._d_out_image.fDiv = 0

        self._d_out_image.pixels = "".join(
            [chr(value) for value in img.flat])

        self.ret_s = self._ImageDataOut.write()
        if not self.ret_s:
            print messege_text_17
        else:
            print messege_text_09

    # =============================================================================================
    # Function Name   :image_data_setting_l
    # Description     :This function is to set the CameraImage type data.
    # Argument        :path information, image data
    # Return          :nothing special
    # =============================================================================================
    def image_data_setting_l(self, path_val, img):
        return_temp = False

        self.image_work_l.height = copy.deepcopy(self.height)
        self.image_work_l.width = copy.deepcopy(self.width)
        self.image_work_l.fDiv = 0
        self.image_work_l.bpp = 24

        return_temp = self.check_extension(path_val)
        self.image_work_l.format = copy.deepcopy(self.imageformat)

        self.image_work_l.pixels = "".join(
            [chr(value) for value in img.flat])

    # =============================================================================================
    # Function Name   :image_data_setting_c
    # Description     :This function is to set the CameraImage type data.
    # Argument        :path information, image data
    # Return          :nothing special
    # =============================================================================================
    def image_data_setting_c(self, path_val, img):
        return_temp = False

        self.image_work_c.height = copy.deepcopy(self.height)
        self.image_work_c.width = copy.deepcopy(self.width)
        self.image_work_c.fDiv = 0
        self.image_work_c.bpp = 24

        return_temp = self.check_extension(path_val)
        self.image_work_c.format = copy.deepcopy(self.imageformat)

        self.image_work_c.pixels = "".join(
            [chr(value) for value in img.flat])

    # =============================================================================================
    # Function Name   :image_data_setting_r
    # Description     :This function is to set the CameraImage type data.
    # Argument        :path information, image data
    # Return          :nothing special
    # =============================================================================================
    def image_data_setting_r(self, path_val, img):
        return_temp = False

        self.image_work_r.height = copy.deepcopy(self.height)
        self.image_work_r.width = copy.deepcopy(self.width)
        self.image_work_r.fDiv = 0
        self.image_work_r.bpp = 24

        return_temp = self.check_extension(path_val)
        self.image_work_r.format = copy.deepcopy(self.imageformat)

        self.image_work_r.pixels = "".join(
            [chr(value) for value in img.flat])

    # =============================================================================================
    # Function Name   :validity_check_of_path
    # Description     :This function to confirmation the validity
    #                 :of the path information.
    # Argument        :Path information
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def validity_check_of_path(self, path_value):
        return_value = False

        if os.path.exists(path_value) is False:
            print messege_text_13

        elif self.check_extension(path_value) is False:
            print messege_text_14

        else:
            return_value = True

        return return_value

    # =============================================================================================
    # Function Name   :check_extension
    # Description     :This function is to confirmation the image format.
    # Argument        :Path information
    # Return          :information of normal end(True) or abnormal end(False).
    # =============================================================================================
    def check_extension(self, value):
        return_value = False

        root, ext = os.path.splitext(value)
        ext = ext.lower()

        if (ext == ".bmp") or (ext == ".dib"):
            self.imageformat = "bitmap"
            return_value = True
        elif (ext == ".jpg") or (ext == ".jpeg") or (ext == ".jpe"):
            self.imageformat = "jpeg"
            return_value = True
        elif (ext == ".png"):
            self.imageformat = "png"
            return_value = True
        elif (ext == ".tiff") or (ext == ".tif"):
            self.imageformat = "tiff"
            return_value = True
        else:
            self.imageformat = "bitmap"

        return return_value

    # =============================================================================================


def ImageLoaderInit(manager):
    profile = OpenRTM_aist.Properties(defaults_str=imageloader_spec)
    manager.registerFactory(profile,
                            ImageLoader,
                            OpenRTM_aist.Delete)


def MyModuleInit(manager):
    ImageLoaderInit(manager)

    # Create a component
    comp = manager.createComponent("ImageLoader")


def main():
    mgr = OpenRTM_aist.Manager.init(sys.argv)
    mgr.setModuleInitProc(MyModuleInit)
    mgr.activateManager()
    mgr.runManager()

if __name__ == "__main__":
    main()
